// Propeller Display with H8S/2633 Microcontroller-Board
// Entry H3210
// Renesas H8 Design Contest 2003
//
// Johann Gysin

#include "ioh82633.h"
// #include "LCD.h" // Debug
#include "SCI.h"
#include "font.c"
#include "Pic_TSU_Logo.c"
#include "Pic_Farbtest.c"
#include "Pic_Smiley.c"
#include "Pic_Beach.c"
#include "Pic_Heart.c"
#include "Pic_Swiss.c"
#include "Pic_Fish.c"
#include "Pic_Sea.c"
#include "Pic_UBoot.c"
#include "Pic_Farbpic.c"
#include "Pic_Muenze.c"

#define enable_irqs()  asm("andc #0x3F,ccr") // Clear I-Bit in
#define disable_irqs() asm("orc #0xC0,ccr")  // Set I-Bit in CCR

#define PictureLengthMax_    2400
#define DisplayLength_       200
#define UserStringLengthMax_ 400

// Global variables
unsigned int Picture[PictureLengthMax_][3];              // Picutre[PixtureLength][Red/Green/Blue]
const unsigned int PictureLengthMax = PictureLengthMax_; // Max PictureLength
int PictureLength;                                       // Actual PictureLength
int PictureIndex;                                        // Actual PictureIndex
const unsigned int DisplayLength = DisplayLength_;       // Count of visible pixels in a round of the propeller
unsigned int timer_tick = 0;
unsigned int seconds = 0;
unsigned int seconds_old = -1;
unsigned char dummy;
unsigned char UserString[UserStringLengthMax_];          // Text to display with the propeller
const int UserString_Length_Max = UserStringLengthMax_;
int UserString_Length = 0;
unsigned char StartString[] = "StarT";      // Startsequency for wireless data transfer
const unsigned char StartString_Length = 5; // Length of the startsequency
unsigned char StartString_Index = 0;        // Actual position in the startsequency
int DisplayPosition;
unsigned char TempString[20];
int ScrollSteps = 3;                        // How many pixels to scroll in a step

// Functions
void Timer_Tick(void);
void StringInit(unsigned char *str, unsigned int str_length);
void AddChar(unsigned char *MainString, unsigned char AddChar); // Add a character to a string
void RF_ProcessData(void);
void DisplayUpdate(void);
void InsertText(unsigned char *str);                  // Inserts a character to the displayable bitmap
void InsertPicture(unsigned char Pic);                // Adds a color-picture to the displayable bitmap
void ResetPicture(void);
void IntToStr(int zahl, unsigned char *str);

// Interrupt-Functions

#pragma interrupt
void Timer_Tick(void)
  {
  timer_tick++;
    
  // 20.8333us 
  if ((timer_tick % 1) == 0)
    {
    DisplayUpdate();
    }
    
  // 1ms 
  if ((timer_tick % 48) == 0)
    {
    // Do something every millisecond...
    }
  
  // 1000ms    
  if ((timer_tick % 48000) == 0)
    {
    timer_tick = 1;
    seconds++;
    }
    
  dummy = TCSR0;
  TCSR0 = TCSR0 & 191; // Clear Compare Match Flag    
  }
  
// Executed by the Photo-Interrupter
#pragma interrupt
void IRQ0(void)
  {    
  TCNT0 = 0;                        // Reset Timer

  PictureIndex = DisplayPosition + DisplayLength;
  
  // Scroll
  DisplayPosition = DisplayPosition + ScrollSteps;
  if (DisplayPosition > PictureLength)
    {
    DisplayPosition = 0 - DisplayLength;
    }
    
  dummy = ISR;
  ISR = ISR & 254;                  // Interrupt 0 rcksetzen
  }

int main (void)
  {
  int counter;
    
  counter = -10;
  
  // LCD-Display init
  //LCD_Init(0);
  
  // SCI init (Serial Interface)
  SCI0_Init();
  SCI2_Init();
  
  // Counter Init
  MSTPCRA = MSTPCRA & 239; // Clear Bit 4 to start timer (TMR0, TMR1)
  TCORA0 = 6;              // (1/(Phi/64))*6 = 20.8333us (Timer to drive LED's)
  TCR0 = 64 | 8 | 2;       // CMFA Interrupt enabled, Clear @ Compare Match A, Internal Clock Phi/64
  
  // Ports
  PEDDR = 255; // All Port E Pins as output
  PDDDR = 255; // All Port D Pins as output
  P1DDR = 192;
  PCDDR = 128;
  
  // Enable Radio PCB
  PGDDR = 4 | 2 | 1;       // IO's as outputs
  PGDR = 1;                // Receive Mode, Channel 0, PWR_Up
  
  // Propeller Display init
  ResetPicture();
  InsertText("Propeller Display      Renesas H8 Design Contest 2003   Entry H3210 ");
  InsertPicture(2);
  DisplayPosition = 0 - DisplayLength;
  PictureIndex    = DisplayPosition + DisplayLength;
  
  // Enable Interrupts
  ISCRL = 2;     // IRQ0 on rising signal
  IER = 1;       // IRQ0 Enable (Photo-Interrupter for picture synchronization with the propeller position)
  enable_irqs(); // Enable IRQ's
  
  while (1)
    {
    RF_ProcessData();
    }
    
  }

void DisplayUpdate(void)
  {
  static int divider = 0;
  static int counter = 0;
  int PictureIndexTemp;
  
  if (PictureIndex > -1)
    {
    if (divider < 3)
      {
      switch (counter)
        {
        case 0: // Red
                PictureIndexTemp = PictureIndex-2;
                if ((PictureIndexTemp > -1) & (PictureIndexTemp < PictureLength))
                  {
                  PCDR = 0;
                  P1DR = 192;
  
                  PEDR = Picture[PictureIndexTemp][0] & 255;         // Lower 8-Bits of the picture
                  PDDR = (Picture[PictureIndexTemp][0] >> 8) & 255;  // Upper 8-Bits of the picture
                  }
                else
                  {
                  PEDR = 0;
                  PDDR = 0;
                  }
                break;
              
        case 1: // Green
                PictureIndexTemp = PictureIndex-1;
                if ((PictureIndexTemp > -1) & (PictureIndexTemp < PictureLength))
                  {
                  PCDR = 128;
                  P1DR = 128;
  
                  PEDR = Picture[PictureIndexTemp][1] & 255;         // Lower 8-Bits of the picture
                  PDDR = (Picture[PictureIndexTemp][1] >> 8) & 255;  // Upper 8-Bits of the picture
                  }
                else
                  {
                  PEDR = 0;
                  PDDR = 0;
                  }
                break;
              
        case 2: // Blue
                PictureIndexTemp = PictureIndex;
                if ((PictureIndexTemp > -1) & (PictureIndexTemp < PictureLength))
                  {
                  PCDR = 128;
                  P1DR = 64;
  
                  PEDR = Picture[PictureIndexTemp][2] & 255;         // Lower 8-Bits of the picture
                  PDDR = (Picture[PictureIndexTemp][2] >> 8) & 255;  // Upper 8-Bits of the picture
                  }
                else
                  {
                  PEDR = 0;
                  PDDR = 0;
                  }
                break;
        }
      }
    else
      {
      PEDR = 0;
      PDDR = 0;
      }
      
    counter++;
    if (counter > 2)
      {
      counter = 0;
      divider++;
      }
    
    if (divider > 7)
      {
      divider = 0;
      if (PictureIndex > -2)
        {
        PictureIndex--;
        }
      }
    }
  }
  
void ResetPicture(void)
  {
  PictureLength = 0;
  }
  
void InsertText(unsigned char *str)
  {
  char NoPicture;
  int index;
  int color;
    
  while (*str != '\0')
    {
      
    // If it is a picture and not a character insert it
    NoPicture = 1;
    switch (*str)
      {
      case 177: InsertPicture(0); // TSU-Logo
                NoPicture = 0;
                break;
      
      case 178: InsertPicture(1); // Colortest
                NoPicture = 0;
                break;
      
      case 179: InsertPicture(2); // Smiley
                NoPicture = 0;
                break;
      
      case 180: InsertPicture(3); // Beach
                NoPicture = 0;
                break;
      
      case 181: InsertPicture(4); // Heart
                NoPicture = 0; 
                break;
      
      case 182: InsertPicture(5); // Swiss Flag
                NoPicture = 0;
                break;
      
      case 183: InsertPicture(6); // Fish
                NoPicture = 0;
                break;
            
      case 184: InsertPicture(7); // Sea
                NoPicture = 0;
                break;
            
      case 185: InsertPicture(8); // submarine
                NoPicture = 0;
                break;
                  
      case 186: for (index=0; index<32; index++)
                  {
                  InsertPicture(9); // Animation: Running man
                  }
                ScrollSteps = 16;
                NoPicture = 0;
                break;
                        
      case 187: for (index=0; index<9; index++)
                  {
                  InsertPicture(10); // Animation: Coin
                  }
                ScrollSteps = 16;
                NoPicture = 0;
                break;
      }
    
    // If it isn't a picture, it's a character  
    if (NoPicture)
      {
      for (index=0; index<5; index++)
        {
        for (color=0; color<3; color++)
          {
          Picture[PictureLength][color] = font[*str][index];
          }
          
        // Update Picture-Length
        if (PictureLength < PictureLengthMax)
          {
          PictureLength++;
          }
        }
      for (color=0; color<3; color++)
        {
        Picture[PictureLength][color] = 0; // Space between the characters
        }
      
      // Update Picture-Length  
      if (PictureLength < PictureLengthMax)
        {
        PictureLength++;
        }
      }
      
    str++;                                 // Next character
    }
    
  DisplayPosition = 0 - DisplayLength;
  }
  
void InsertPicture(unsigned char Pic)
  {
  int index;
  int color;
    
  switch (Pic)
    {
    // TSU-Logo
    case 0 :  for (index=0; index<Pic_TSU_Logo_Length; index++)
                {
                for (color=0; color<3; color++)
                  {
                  Picture[PictureLength][color] = Pic_TSU_Logo[index][color];
                  }
                // Actualize picture-length  
                if (PictureLength < PictureLengthMax)
                  {
                  PictureLength++;
                  }
                }
              break;
   
    // All colors            
    case 1 :  for (index=0; index<Pic_Farbtest_Length; index++)
                {
                for (color=0; color<3; color++)
                  {
                  Picture[PictureLength][color] = Pic_Farbtest[index][color];
                  }
                // Actualize picture-length  
                if (PictureLength < PictureLengthMax)
                  {
                  PictureLength++;
                  }
                }
              break;
     
    // Smiley           
    case 2 :  for (index=0; index<Pic_Smiley_Length; index++)
                {
                for (color=0; color<3; color++)
                  {
                  Picture[PictureLength][color] = Pic_Smiley[index][color];
                  }
                // Actualize picture-length  
                if (PictureLength < PictureLengthMax)
                  {
                  PictureLength++;
                  }
                }
              break;
       
    // Beach   
    case 3 :  for (index=0; index<Pic_Beach_Length; index++)
                {
                for (color=0; color<3; color++)
                  {
                  Picture[PictureLength][color] = Pic_Beach[index][color];
                  }
                // Actualize picture-length  
                if (PictureLength < PictureLengthMax)
                  {
                  PictureLength++;
                  }
                }
              break;
       
    // Heart   
    case 4 :  for (index=0; index<Pic_Heart_Length; index++)
                {
                for (color=0; color<3; color++)
                  {
                  Picture[PictureLength][color] = Pic_Heart[index][color];
                  }
                // Actualize picture-length  
                if (PictureLength < PictureLengthMax)
                  {
                  PictureLength++;
                  }
                }
              break;
       
    // Swiss Flag   
    case 5 :  for (index=0; index<Pic_Swiss_Length; index++)
                {
                for (color=0; color<3; color++)
                  {
                  Picture[PictureLength][color] = Pic_Swiss[index][color];
                  }
                // Actualize picture-length  
                if (PictureLength < PictureLengthMax)
                  {
                  PictureLength++;
                  }
                }
              break;
                
    // Fish   
    case 6 :  for (index=0; index<Pic_Fish_Length; index++)
                {
                for (color=0; color<3; color++)
                  {
                  Picture[PictureLength][color] = Pic_Fish[index][color];
                  }
                // Actualize picture-length  
                if (PictureLength < PictureLengthMax)
                  {
                  PictureLength++;
                  }
                }
              break;
                
    // Sea   
    case 7 :  for (index=0; index<Pic_Sea_Length; index++)
                {
                for (color=0; color<3; color++)
                  {
                  Picture[PictureLength][color] = Pic_Sea[index][color];
                  }
                // Actualize picture-length  
                if (PictureLength < PictureLengthMax)
                  {
                  PictureLength++;
                  }
                }
              break;
                
    // Submarine   
    case 8 :  for (index=0; index<Pic_UBoot_Length; index++)
                {
                for (color=0; color<3; color++)
                  {
                  Picture[PictureLength][color] = Pic_UBoot[index][color];
                  }
                // Actualize picture-length  
                if (PictureLength < PictureLengthMax)
                  {
                  PictureLength++;
                  }
                }
              break;
                                
    // Animation: Running man
    case 9 :  for (index=0; index<Pic_Farbpic_Length; index++)
                {
                for (color=0; color<3; color++)
                  {
                  Picture[PictureLength][color] = Pic_Farbpic[index][color];
                  }
                // Actualize picture-length  
                if (PictureLength < PictureLengthMax)
                  {
                  PictureLength++;
                  }
                }
              break;
                                    
    // Animation: Rotating coin
    case 10:  for (index=0; index<Pic_Muenze_Length; index++)
                {
                for (color=0; color<3; color++)
                  {
                  Picture[PictureLength][color] = Pic_Muenze[index][color];
                  }
                // Actualize picture-length  
                if (PictureLength < PictureLengthMax)
                  {
                  PictureLength++;
                  }
                }
              break;
    }
  
  DisplayPosition = 0 - DisplayLength;
  }
  
void RF_ProcessData(void)
  {
  unsigned char character;
  int index;
  int BufCharacterCount;
  static unsigned char synced = 0; // 1 => Startsequency was ok
    
  BufCharacterCount = SCI0_GetRecBufferCount();
  if (BufCharacterCount > 0)
    {
    for (index=0; index<BufCharacterCount; index++)
      {
      character = SCI0_GetRecBufferChar(); // Get a character from the receive-buffer
      if (synced)
        {
        if ((character == 13) | (character == 10)) // If end of string reached...
          {
          AddChar(UserString, '\0');
          ResetPicture();
          ScrollSteps = 3;
          InsertText(UserString);
          synced = 0;
          }
        else
          {
          if (UserString_Length < (UserString_Length_Max-1)) // If string isn't full...
            {
            AddChar(UserString, character); // Add character to string
            UserString_Length++;
            }
          else
            {
            // If string is full, display the current picture and ignore the rest
            AddChar(UserString, '\0');
            ResetPicture();
            InsertText(UserString);
            synced = 0;
            }
          }
        }
      else
        {
        if (character == StartString[StartString_Index])
          {
          StartString_Index++;
          if (StartString_Index >= StartString_Length) // If startsequency is complete...
            {
            StartString_Index = 0;
            StringInit(UserString, UserString_Length_Max);
            UserString_Length = 0;
            synced = 1;
            PictureLength = 0;
            }
          }
        else
          {
          StartString_Index = 0;
          }
        }  
      }  
    }
  }

void AddChar(unsigned char *MainString, unsigned char AddChar)
  {
  if (*MainString != '\0')       // If first character isn't null...
    {  
    do
      {
      MainString++;
      }
    while (*MainString != '\0'); // ...do until a null is found
    }
    
  *MainString = AddChar;         // Insert the character at the position of the found null
  MainString++;                  // Increment pointer
  *MainString = '\0';            // Insert null at the end of the string
  }
  
void StringInit(unsigned char *str, unsigned int str_length)
  {
  unsigned int index;
    
  // Fill string with null's
  for (index=0; index<str_length; index++)
    {
    str[index] = '\0';
    }
  }
  
void IntToStr(int zahl, unsigned char *str)
  {
  unsigned char ziffer;
  unsigned char flag;
  unsigned char str_zahl;
  unsigned int maske;
    
  if (zahl < 0)
    {
    *str = '-';                                  // If the number is negative, write a -
    str++;
    zahl = 32768 - (zahl & ~32768);              // Take absolutely value (remove -)
    }
    
  if (zahl == 0)
    {
    *str = '0';                                  // If the number is 0, output this directly
    str++;
    }
  else
    {
    maske = 10000;                               // Set mask to the first digit of the 16-bit integer value
    flag = 0;                                    // Suppress leading zeroes
      
    while (maske > 0)
      {
      ziffer = zahl / maske;                     // Calculate the value of a single digit
        
      if (ziffer != 0)
        {
        flag = 1;                                // On the first number disable the suppression of leading zeroes
        }
        
      if (flag == 1)
        {
        str_zahl = '0' + ziffer;                 // Calculate ASCII-value of the digit
        *str = str_zahl;                         // Write the digit
        str++;
        }
      zahl = zahl - (maske * ziffer);            // Subtract the last digit from the value
      maske = maske / 10;                        // Set mask to the next digit
      }
    }
  }
