// LCD-Routines for a 4-line display with 40 characters and HD44780U controller
// Johann Gysin

#include "ioh82633.h"
#include "lcd.h"

unsigned char DDRAM_Offset_Line_1;
unsigned char DDRAM_Offset_Line_2;
unsigned char DDRAM_Offset_Line_3;
unsigned char DDRAM_Offset_Line_4;
unsigned char ControllerCount;

void LCD_Wait_Time(int Time);
void LCD_Wait(unsigned char Enable);
void LCD_Init(unsigned char DisplayType);
void LCD_WriteInstr(unsigned char LCD_Controller, unsigned char Instr);
void LCD_Clear(void);
void LCD_Control(unsigned char LCD_Controller, unsigned char disp_on, unsigned char cursor_on, unsigned char blink);
char LCD_SetCursor(unsigned char line, unsigned char character);
void LCD_WriteChar(unsigned char LCD_Controller, unsigned char Data);
void LCD_WriteStr(unsigned char line, unsigned char character, const unsigned char *str);
void LCD_WriteInt(unsigned char line, unsigned char character, int number);

void LCD_Wait_Time(int Time)
  {
  int a;
  int b;

  for (a=0; a<Time; a++)
    {
    for (b=0; b<3650; b++) // wait about 1ms
      {
      }
    }
  }
  
void LCD_Wait(unsigned char Enable)
  {
  unsigned char busy;
  
  PBDDR = 8 | 4 | 2 | 1; // Set all pins on port B as outputs, except D4 to D7
  do
    {
    PBDR = 4;            // Read from Display
    PBDR = Enable | 4;   // Read and Enable (High Nibble)
    busy = PORTB & 128;  // Read Busy Flag
    PBDR = 4;            // Enable low
    PBDR = Enable | 4;   // Read and Enable (Low Nibble)
    PBDR = 4;            // Enable low
    }
  while (busy);          // Repeat until Busy-Flag is reset
  PBDR = 0;              // Disable / Write
  PBDDR = 255;           // All pins as outputs
  }

void LCD_Init(unsigned char DisplayType)
  {
  unsigned char enable;
    
  enable = 9;         // Init both display controllers (bit 0 and bit 3)
    
  PBDDR = 255;        // Pord B as output
    
  LCD_Wait_Time(15);  // Wait min. 15ms
  PBDR = enable;      // Enable1, Enable2
  PBDR = enable | 48; // Enable1, Enable2, Data (3), Init
  PBDR = 48;          // Data (3)
  LCD_Wait_Time(5);   // Wait min. 5ms
  PBDR = enable;      // Enable1, Enable2
  PBDR = enable | 48; // Enable1, Enable2, Data (3), Init
  PBDR = 48;          // Data (3)
  LCD_Wait_Time(1);   // Wait min. 160us
  PBDR = enable;      // Enable1, Enable2
  PBDR = enable | 48; // Enable1, Enable2, Data (3), Init
  PBDR = 48;          // Data (3)
  LCD_Wait_Time(1);   // Wait min. 160us
  PBDR = enable;      // Enable1, Enable2
  PBDR = enable | 32; // Enable1, Enable2, Data (2), 4-Bit Mode
  PBDR = 32;          // Data (2)
  LCD_Wait_Time(1);   // Wait min. 160us
    
  // From now, 2 Nibbles required
  LCD_WriteInstr(3, 40);      // DataLength 4-Bit, 2 Lines, 5x8Dots Font
  LCD_Clear();                // Clear Display
  LCD_Control(3, 1, 0, 0);    // Display On, Cursor Off, Blink Off
  LCD_WriteInstr(3, 128 | 0); // Set DDRAM Addr. 0

  LCD_Wait_Time(1);           // Wait
  
  // Set variables for different display types
  switch (DisplayType)
    {
    // Display with 4 lines and 40 chracters
    case 0: DDRAM_Offset_Line_1 = 0;
            DDRAM_Offset_Line_2 = 64;
            DDRAM_Offset_Line_3 = 0;
            DDRAM_Offset_Line_4 = 64;
            ControllerCount = 2;
            break;
    
    // Display with 4 lines and 20 chracters
    case 1: DDRAM_Offset_Line_1 = 0;
            DDRAM_Offset_Line_2 = 64;
            DDRAM_Offset_Line_3 = 20;
            DDRAM_Offset_Line_4 = 84;
            ControllerCount = 1;
    }
  }

void LCD_WriteInstr(unsigned char LCD_Controller, unsigned char Instr)
  {
  unsigned char enable;
  
  switch (LCD_Controller)
    {
    case 1: enable = 8; // Activates first LCD_Controller on the display
            break;
    
    case 2: enable = 1; // Activates second LCD_Controller on the display (only on large displays)
            break;
    
    case 3: enable = 9; // Activates both LCD_Controller
            break;
    
    default: enable = 8;
    }
    
  // Send High Nibble
  PBDR = enable;                 // Enable1, Enable2
  PBDR = enable | (240 & Instr); // Enable and HighNibble from Instr
  PBDR = 240 & Instr;            // HighNibble from Instr
  LCD_Wait(enable);              // wait 37us (?) 
    
  // Send Low Nibble
  Instr = Instr << 4;
  PBDR = enable;                 // Enable1, Enable2
  PBDR = enable | Instr;         // Enable and LowNibble from Instr
  PBDR = Instr;                  // LowNibble from Instr
  LCD_Wait(enable);              // wait 37us (?)
  }

void LCD_Clear(void)
  {
  LCD_WriteInstr(3, 1);
  }
  
void LCD_Control(unsigned char LCD_Controller, unsigned char disp_on, unsigned char cursor_on, unsigned char blink)
  {
  if (disp_on != 0)
    {
    disp_on = 4;
    }
  
  if (cursor_on != 0)
    {
    cursor_on = 2;
    }
    
  if (blink != 0)
    {
    blink = 1;
    }
    
  LCD_WriteInstr(LCD_Controller, 8 | disp_on | cursor_on | blink);
  }
  
char LCD_SetCursor(unsigned char line, unsigned char character)
  {
  unsigned char LCD_Controller = 1;
  unsigned char LCD_DDRAM_Address = 0;
    
  switch (line)
    {
    case 1: LCD_Controller = 1;
            LCD_DDRAM_Address = character-1 + DDRAM_Offset_Line_1;
            break;
    
    case 2: LCD_Controller = 1;
	    LCD_DDRAM_Address = character-1 + DDRAM_Offset_Line_2;
            break;
    
    case 3: if (ControllerCount == 1)
              {
              LCD_Controller = 1;
              }
            else
              {
              LCD_Controller = 2;
	      }
            LCD_DDRAM_Address = character-1 + DDRAM_Offset_Line_3;
            break;
    
    case 4: if (ControllerCount == 1)
              {
              LCD_Controller = 1;
              }
            else
              {
              LCD_Controller = 2;
	      }
	    LCD_DDRAM_Address = character-1 + DDRAM_Offset_Line_4;
            break;
    }
  
  // X-Position define with DDRAM-Address
  LCD_WriteInstr(LCD_Controller, 128 | LCD_DDRAM_Address);

  return(LCD_Controller);    
  }

void LCD_WriteChar(unsigned char LCD_Controller, unsigned char Data)
  {
  unsigned char RegisterSelect;
  unsigned char Enable;
  unsigned char Converted;

  // Convert characters
  Converted = Data;
  switch (Data)
    {
    case 228: Converted = 225; // 
              break;

    case 246: Converted = 239; // 
              break;

    case 252: Converted = 245; // 
              break;

    case 196: Converted = 225; // 
              break;

    case 214: Converted = 239; // 
              break;

    case 220: Converted = 245; // 
              break;
    }
  Data = Converted;
  
  if ((Data < 16) | (Data > 31)) // Only output printable characters
    { 
    RegisterSelect = 2; // DDRAM
  
    // select LCD_Controller	  
    switch (LCD_Controller)
      {
      case 1: Enable = 8;
              break;

      case 2: Enable = 1;
              break;

      case 3: Enable = 9;
              break;

      default: Enable = 8;
      }

    // Send High Nibble
    PBDR = Enable | RegisterSelect;
    PBDR = Enable | (240 & Data) | RegisterSelect;
    PBDR = 240 & (Data | RegisterSelect);
    LCD_Wait(Enable);

    // Send Low Nibble
    Data = Data << 4;
    PBDR = Enable | RegisterSelect;
    PBDR = Enable | Data | RegisterSelect;
    PBDR = Data | RegisterSelect;
    LCD_Wait(Enable);
    }
  }
  
void LCD_WriteStr(unsigned char line, unsigned char character, const unsigned char *str)
  {
  unsigned char LCD_Controller;
    
  LCD_Controller = LCD_SetCursor(line, character); // Set cursor to defined line/character position  
    
  while (*str != '\0')
    {
    LCD_WriteChar(LCD_Controller, *str); // Output charater to the display
    str++;              
    }  
  }
  
void LCD_WriteInt(unsigned char line, unsigned char character, int number)
  {
  unsigned char digit;
  unsigned char flag;
  unsigned char LCD_Controller;
  unsigned char str_number;
  unsigned int mask;
  
  LCD_Controller = LCD_SetCursor(line, character); // Put the cursor to startposition
    
  if (number < 0)
    {
    LCD_WriteChar(LCD_Controller, '-');          // If the number is negative, write a -
    number = 32768 - (number & ~32768);          // Take absolutely value (remove -) 
    }
    
  if (number == 0)
    {
    LCD_WriteChar(LCD_Controller, '0');          // If the number is 0, output this directly
    }
  else
    {
    mask = 10000;                                // Set mask to the first digit of the 16-bit integer value
    flag = 0;                                    // Suppress leading zeroes
      
    while (mask > 0)
      {
      digit = number / mask;                     // Calculate the value of a single digit
        
      if (digit != 0)
        {
        flag = 1;                                // On the first number disable the suppression of leading zeroes
        }
        
      if (flag == 1)
        {
        str_number = '0' + digit;                 // Calculate ASCII-value of the digit
        LCD_WriteChar(LCD_Controller, str_number); // Write the digit to the display
        }
      number = number - (mask * digit);           // Subtract the last digit from the value
      mask = mask / 10;                           // Set mask to the next digit
      }
    }
  }
