/* Linker Commands for H8S 2633 in Mode 7
 * Original from Johan Henning
 * Little modifications by Johann Gysin
 * 28.01.2002
 */
/*
 *  Linker command file for stand alone operation
 *
 *
 *  Memory Map:
 *
 *  H 000000    +-------------------+     \
 *              |     Vectors       |      |
 *  H 000200    +-------------------+
 *              |                   |      |- 256 KByte
 *              |      FLASH        |
 *              |                   |      |
 *  H 040000    +-------------------+     /
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *              |                   |
 *  H FFB000    +-------------------+
 *              |                   |     \
 *              |   Internal RAM    |      |- 15 + 15/16 KByte
 *              |                   |     /
 *  H FFEFBF    +-------------------+
 *              |                   |
 *  H FFF800    +-------------------+
 *              |                   |
 *              |Internal Registers |
 *              |                   |
 *  H FFFF3F    +-------------------+
 *              |                   |
 *  H FFFF60    +-------------------+
 *              |Internal Registers |
 *  H FFFFC0    +-------------------+
 *              |   Internal RAM    |         1/16 KByte
 *  H FFFFFF    +-------------------+
 *
 */
SEARCH_DIR (C:\cygnus\lib\H8300s\)
OUTPUT_FORMAT("srec")
OUTPUT_ARCH(h8300s)

MEMORY
{
  rom   : o = 0x0000000, l = 128k       /* code space */
  ram   : o = 0x0ffb000, l = 0x03fc0    /* data and stack space */
  tiny  : o = 0x0ffffc0, l = 0x040      /* data space */
}

SECTIONS 
{
  .vectors :
    {
    _vect = .;
    *(.vect);
    *(.progid);
    *(.monitor);
    } > rom

  .tors :
    {
    ___ctors = . ;
    *(.ctors)
    ___ctors_end = . ;
    ___dtors = . ;
    *(.dtors)
    ___dtors_end = . ;
    } > rom

  .text :
    {
    *(.text)
    *(.rodata)
     _etext = ALIGN(16);
     } > rom

  .data : AT(_etext)    /* places a ROM image at _etext */
    {
    _data = .;
    *(.data)
    _edata = .; 
    } > ram

  .bss (NOLOAD) :
    {
    _bss = . ;
    *(.bss)
    *(COMMON)
    _ebss = .;
    _end = .;  
    } > ram

/* .uir stands for un-initialized-ram  */
  .uir :
    {
    _uir  = .;
    *(.uir)
    _euir = .;
    } > ram

  .stack (NOLOAD) :
    {
    _stack_last  = .;  /* deepest location of the stack                              */
    . += 0x080;        /* reserve 128 bytes stack space for Cstart.asm and main.cpp  */
    _stack = .; 
    _stack_first = .;  /* shallowest location of the stack                           */
    *(.stack)
    } > ram

  .stab 0 (NOLOAD) :
    {
    [ .stab ]
    }
  .stabstr 0 (NOLOAD) :
    {
    [ .stabstr ]
    }
}
