// SCI-Routines for the serial interfaces (SCI0 and SCI2)
// Johann Gysin

#include "ioh82633.h"
#include "SCI.h"

#define SCI0_RecBuf_Length 256 // Length of the receivebuffer of SCI0
#define SCI0_SenBuf_Length 256 // Length of the sendbuffer of SCI0

#define SCI2_RecBuf_Length 256 // Length of the receivebuffer of SCI2
#define SCI2_SenBuf_Length 256 // Length of the sendbuffer of SCI2

// Forward declarations
void SCI0_Init(void);
void SCI0_WriteChar(unsigned char Data);
void SCI0_WriteStr(const unsigned char *str);
void SCI0_WriteLn(const unsigned char *str);
void SCI0_WriteCRLF(void);
void SCI0_WriteInt(int zahl);
void SCI0_IRQRecChar(void);
int SCI0_GetRecBufferCount(void);
int SCI0_GetSenBufferCount(void);
char SCI0_GetRecBufferChar(void);
void SCI0_PutSenBufferChar(unsigned char character);
void SCI0_IRQSenChar(void);
void SCI0_IRQError(void);

void SCI2_Init(void);
void SCI2_WriteChar(unsigned char Data);
void SCI2_WriteStr(const unsigned char *str);
void SCI2_WriteLn(const unsigned char *str);
void SCI2_WriteCRLF(void);
void SCI2_WriteInt(int zahl);
void SCI2_IRQRecChar(void);
int SCI2_GetRecBufferCount(void);
int SCI2_GetSenBufferCount(void);
char SCI2_GetRecBufferChar(void);
void SCI2_PutSenBufferChar(unsigned char character);
void SCI2_IRQSenChar(void);
void SCI2_IRQError(void);

// Global Variables
unsigned char dummy;

unsigned char SCI0_RecBuf[SCI0_RecBuf_Length];
unsigned int SCI0_RecBuf_CharCount;
unsigned int SCI0_RecBuf_ReadPos = 0;
unsigned int SCI0_RecBuf_WritePos = 0;
unsigned char SCI0_SenBuf[SCI0_SenBuf_Length];
unsigned int SCI0_SenBuf_CharCount;
unsigned int SCI0_SenBuf_ReadPos = 0;
unsigned int SCI0_SenBuf_WritePos = 0;

unsigned char SCI2_RecBuf[SCI2_RecBuf_Length];
unsigned int SCI2_RecBuf_CharCount;
unsigned int SCI2_RecBuf_ReadPos = 0;
unsigned int SCI2_RecBuf_WritePos = 0;
unsigned char SCI2_SenBuf[SCI2_SenBuf_Length];
unsigned int SCI2_SenBuf_CharCount;
unsigned int SCI2_SenBuf_ReadPos = 0;
unsigned int SCI2_SenBuf_WritePos = 0;

void SCI0_Init(void) // Like example in hardware manual page 621
  {
  unsigned int index;
  unsigned int wait;
  unsigned char Clock;
  
  SCI0_RecBuf_CharCount = 0;
  SCI0_SenBuf_CharCount = 0;
  
  // Initialize receivebuffer    
  for (index=0; index<SCI0_RecBuf_Length; index++)
    {
    SCI0_RecBuf[index] = '\0';
    }
    
  // Initialize sendbuffer
  for (index=0; index<SCI0_SenBuf_Length; index++)
    {
    SCI0_SenBuf[index] = '\0';
    }  
  
  Clock = 0; // SCI Clock = Internal (Asynchronous Mode)
  
  MSTPCRB = MSTPCRB & 127;         // SCI0 Start (Bit 7 => 0)
  SCR0 = SCR0 & 207;               // Reset TE- and RE-Bits
  SCR0 = (SCR0 & 207) | Clock;     // Set Clock
  SMR0 = 0;                        // Asynch, 8-Bit, No Parity, Even Parity, 1 Stop Bit, No Multiproc., Clock = Phi
  SCMR0 = 0;                       // Normal SCI Operation (No Smart Card)
  BRR0 = 29;                       // 19200 Baud => n = 0, N = 29
  for (wait=0; wait<2000; wait++); // Wait min. a 1-Bit-Interval
  //SCR0 = 128 | 64 | 32 | 16;       // Send IRQ, Receive IRQ, Rx/Tx Enabled, Asynch. Clock
  SCR0 = 64 | 32 | 16;             // Receive IRQ, Rx/Tx Enabled, Asynch. Clock
  }

void SCI2_Init(void) // Like example in hardware manual page 621
  {
  unsigned int index;
  unsigned int wait;
  unsigned char Clock;
    
  SCI2_RecBuf_CharCount = 0;
  SCI2_SenBuf_CharCount = 0;
  
  // Initialize receivebuffer    
  for (index=0; index<SCI2_RecBuf_Length; index++)
    {
    SCI2_RecBuf[index] = '\0';
    }
    
  // Initialize sendbuffer
  for (index=0; index<SCI2_SenBuf_Length; index++)
    {
    SCI2_SenBuf[index] = '\0';
    }  
    
  Clock = 0; // SCI Clock = Internal (Asynchronous Mode)
  
  MSTPCRB = MSTPCRB & 223;         // SCI2 Start (Bit 7 => 0)
  SCR2 = SCR2 & 207;               // Reset TE- and RE-Bits
  SCR2 = (SCR2 & 207) | Clock;     // Set Clock
  SMR2 = 0;                        // Asynch, 8-Bit, No Parity, Even Parity, 1 Stop Bit, No Multiproc., Clock = Phi
  SCMR2 = 0;                       // Normal SCI Operation (No Smart Card)
  BRR2 = 29;                       // 19200 Baud => n = 0, N = 29
  for (wait=0; wait<2000; wait++); // Wait min. a 1-Bit-Interval
  //SCR2 = 128 | 64 | 32 | 16;       // Send IRQ, Receive IRQ, Rx/Tx Enabled, Asynch. Clock
  SCR2 = 64 | 32 | 16;             // Receive IRQ, Rx/Tx Enabled, Asynch. Clock
  }
  
void SCI0_WriteChar(unsigned char Data)
  {
  while ((SSR0 & 128) != 128); // Waits until last character is sent
  TDR0 = Data;                 // Write Data to the send register
  dummy = SSR0;                // Dummy read
  SSR0 = SSR0 & 127;           // Data is ready, start sending
  //SCI2_PutSenBufferChar(Data); // Write to sendbuffer
  }

void SCI2_WriteChar(unsigned char Data)
  {
  while ((SSR2 & 128) != 128); // Waits until last character is sent
  TDR2 = Data;                 // Write Data to the send register
  dummy = SSR2;                // Dummy read
  SSR2 = SSR2 & 127;           // Data is ready, start sending
  //SCI2_PutSenBufferChar(Data); // Write to sendbuffer
  }
  
void SCI0_WriteStr(const unsigned char *str)
  {
  while (*str != '\0')
    {
    SCI0_WriteChar(*str);
    str++;              
    }  
  }

void SCI2_WriteStr(const unsigned char *str)
  {
  while (*str != '\0')
    {
    SCI2_WriteChar(*str);
    str++;              
    }  
  }
  
void SCI0_WriteLn(const unsigned char *str)
  {
  while (*str != '\0')
    {
    SCI0_WriteChar(*str);
    str++;              
    }
  SCI0_WriteCRLF();    
  }
  
void SCI2_WriteLn(const unsigned char *str)
  {
  while (*str != '\0')
    {
    SCI2_WriteChar(*str);
    str++;              
    }
  SCI2_WriteCRLF();    
  }
  
void SCI0_WriteCRLF(void)
  {
  SCI0_WriteChar(13); // CR (Carriage Return)
  SCI0_WriteChar(10); // LF (Line Feed)
  }

void SCI2_WriteCRLF(void)
  {
  SCI2_WriteChar(13); // CR (Carriage Return)
  SCI2_WriteChar(10); // LF (Line Feed)
  }
  
void SCI0_WriteInt(int zahl)
  {
  unsigned char digit;
  unsigned char flag;
  unsigned char str_zahl;
  unsigned int mask;
    
  if (zahl < 0)
    {
    SCI0_WriteChar('-');                         // If the number is negative, write a -
    zahl = 32768 - (zahl & ~32768);              // Take absolutely value (remove -) 
    }
    
  if (zahl == 0)
    {
    SCI0_WriteChar('0');                         // If the number is 0, output this directly
    }
  else
    {
    mask = 10000;                                // Set mask to the first digit of the 16-bit integer value
    flag = 0;                                    // Suppress leading zeroes
      
    while (mask > 0)
      {
      digit = zahl / mask;                       // Calculate the value of a single digit
        
      if (digit != 0)
        {
        flag = 1;                                // On the first number disable the suppression of leading zeroes
        }
        
      if (flag == 1)
        {
        str_zahl = '0' + digit;                  // Calculate ASCII-value of the digit
        SCI0_WriteChar(str_zahl);                // Write the digit
        }
      zahl = zahl - (mask * digit);              // Subtract the last digit from the value
      mask = mask / 10;                          // Set mask to the next digit
      }
    }
  }
  
void SCI2_WriteInt(int zahl)
  {
  unsigned char digit;
  unsigned char flag;
  unsigned char str_zahl;
  unsigned int mask;
    
  if (zahl < 0)
    {
    SCI2_WriteChar('-');                         // If the number is negative, write a -
    zahl = 32768 - (zahl & ~32768);              // Take absolutely value (remove -) 
    }
    
  if (zahl == 0)
    {
    SCI2_WriteChar('0');                         // If the number is 0, output this directly
    }
  else
    {
    mask = 10000;                                // Set mask to the first digit of the 16-bit integer value
    flag = 0;                                    // Suppress leading zeroes
      
    while (mask > 0)
      {
      digit = zahl / mask;                       // Calculate the value of a single digit
        
      if (digit != 0)
        {
        flag = 1;                                // On the first number disable the suppression of leading zeroes
        }
        
      if (flag == 1)
        {
        str_zahl = '0' + digit;                  // Calculate ASCII-value of the digit
        SCI2_WriteChar(str_zahl);                // Write the digit to the display
        }
      zahl = zahl - (mask * digit);              // Subtract the last digit from the value
      mask = mask / 10;                          // Set mask to the next digit
      }
    }
  }
  
#pragma interrupt
void SCI0_IRQRecChar(void)
  {
  // Write a character to the buffy when it isn't full
  if (SCI0_RecBuf_CharCount < SCI0_RecBuf_Length)
    {
    SCI0_RecBuf[SCI0_RecBuf_WritePos] = RDR0;
    SCI0_RecBuf_WritePos++;
    SCI0_RecBuf_WritePos = SCI0_RecBuf_WritePos % SCI0_RecBuf_Length; // Go back to the beginning of the buffer
    SCI0_RecBuf_CharCount++;
    }
  dummy = SSR0;
  SSR0 = SSR0 & 191; // RDRF (Receive Data Register Full) reset flag
  }

#pragma interrupt
void SCI2_IRQRecChar(void)
  {
  // Write a character to the buffy when it isn't full
  if (SCI2_RecBuf_CharCount < SCI2_RecBuf_Length)
    {
    SCI2_RecBuf[SCI2_RecBuf_WritePos] = RDR2;
    SCI2_RecBuf_WritePos++;
    SCI2_RecBuf_WritePos = SCI2_RecBuf_WritePos % SCI2_RecBuf_Length; // Go back to the beginning of the buffer
    SCI2_RecBuf_CharCount++;
    }
  dummy = SSR2;
  SSR2 = SSR2 & 191; // RDRF (Receive Data Register Full) reset flag
  }
  
int SCI0_GetRecBufferCount(void)
  {
  return(SCI0_RecBuf_CharCount);
  }
  
int SCI2_GetRecBufferCount(void)
  {
  return(SCI2_RecBuf_CharCount);
  }
  
int SCI0_GetSenBufferCount(void)
  {
  return(SCI0_SenBuf_CharCount);
  }
  
int SCI2_GetSenBufferCount(void)
  {
  return(SCI2_SenBuf_CharCount);
  }
  
char SCI0_GetRecBufferChar(void)
  {
  char character;
  
  character = '\0';
    
  if (SCI0_RecBuf_CharCount > 0)
    {
    character = SCI0_RecBuf[SCI0_RecBuf_ReadPos];
    SCI0_RecBuf_ReadPos++;
    SCI0_RecBuf_ReadPos = SCI0_RecBuf_ReadPos % SCI0_RecBuf_Length;
    SCI0_RecBuf_CharCount--;
    }
    
  return(character);
  }
  
char SCI2_GetRecBufferChar(void)
  {
  char character;
  
  character = '\0';
    
  if (SCI2_RecBuf_CharCount > 0)
    {
    character = SCI2_RecBuf[SCI2_RecBuf_ReadPos];
    SCI2_RecBuf_ReadPos++;
    SCI2_RecBuf_ReadPos = SCI2_RecBuf_ReadPos % SCI2_RecBuf_Length;
    SCI2_RecBuf_CharCount--;
    }
    
  return(character);
  }
  
void SCI0_PutSenBufferChar(unsigned char character)
  {
  if (SCI0_SenBuf_CharCount < SCI0_SenBuf_Length)
    {
    SCI0_SenBuf[SCI0_SenBuf_WritePos] = character;
    SCI0_SenBuf_CharCount++;
    SCI0_SenBuf_WritePos++;
    SCI0_SenBuf_WritePos = SCI0_SenBuf_WritePos % SCI0_SenBuf_Length;
    }
  }
  
void SCI2_PutSenBufferChar(unsigned char character)
  {
  if (SCI2_SenBuf_CharCount < SCI2_SenBuf_Length)
    {
    SCI2_SenBuf[SCI2_SenBuf_WritePos] = character;
    SCI2_SenBuf_CharCount++;
    SCI2_SenBuf_WritePos++;
    SCI2_SenBuf_WritePos = SCI2_SenBuf_WritePos % SCI2_SenBuf_Length;
    }
  }
  
#pragma interrupt
void SCI0_IRQSenChar(void)
  {
  if (SCI0_SenBuf_CharCount > 0)   // If data in sendbuffer
    {
    TDR0 = SCI0_SenBuf[SCI0_SenBuf_ReadPos];
    //SSR2 = SSR2 & 127;           // Data Ready
    SCI0_SenBuf_CharCount--;
    SCI0_SenBuf_ReadPos++;
    SCI0_SenBuf_ReadPos = SCI0_SenBuf_ReadPos % SCI0_SenBuf_Length;
    } 
  dummy = SSR0;
  SSR0 = SSR0 & 127; // Reset TDRE Flag(Transmit Data Register Empty)
  }
  
#pragma interrupt
void SCI2_IRQSenChar(void)
  {
  if (SCI2_SenBuf_CharCount > 0)   // If data in sendbuffer
    {
    TDR2 = SCI2_SenBuf[SCI2_SenBuf_ReadPos];
    //SSR2 = SSR2 & 127;           // Data Ready
    SCI2_SenBuf_CharCount--;
    SCI2_SenBuf_ReadPos++;
    SCI2_SenBuf_ReadPos = SCI2_SenBuf_ReadPos % SCI2_SenBuf_Length;
    } 
  dummy = SSR2;
  SSR2 = SSR2 & 127; // Reset TDRE Flag(Transmit Data Register Empty)
  }

#pragma interrupt  
void SCI0_IRQError(void)
  {
  dummy = SSR0;
  SSR0 = SSR0 & 192; // Clear Errors
  }

#pragma interrupt  
void SCI2_IRQError(void)
  {
  dummy = SSR2;
  SSR2 = SSR2 & 192; // Clear Errors
  }
